@@ -23,7 +23,9 @@
 #include "parser/backtraceparser.h"
 
 #include "aboutbugreportingdialog.h"
+#include "assistantpage_bugzilla_supported_entities.h"
 #include "assistantpage_bugzilla_version.h"
+#include "assistantpage_conclusion.h"
 #include "crashedapplication.h"
 #include "reportassistantpages_base.h"
 #include "reportassistantpages_bugzilla.h"
@@ -57,18 +59,22 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
         auto *m_introduction = new IntroductionPage(this);
 
         auto *m_introductionPage = new KPageWidgetItem(m_introduction, QLatin1String(PAGE_INTRODUCTION_ID));
-        m_pageWidgetMap.insert(QLatin1String(PAGE_INTRODUCTION_ID), m_introductionPage);
         m_introductionPage->setHeader(i18nc("@title", "Welcome to the Reporting Assistant"));
         m_introductionPage->setIcon(QIcon::fromTheme(QStringLiteral("tools-report-bug")));
         m_pageItems.push_back(m_introductionPage);
     }
 
+    // Version check page
+    auto *versionPage = new BugzillaVersionPage(this);
+
+    // Product and version are valid page
+    auto *entityPage = new BugzillaSupportedEntitiesPage(this);
+
     //-Bug Awareness Page
     auto *m_awareness = new BugAwarenessPage(this);
     connectSignals(m_awareness);
 
     auto *m_awarenessPage = new KPageWidgetItem(m_awareness, QLatin1String(PAGE_AWARENESS_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_AWARENESS_ID), m_awarenessPage);
     m_awarenessPage->setHeader(i18nc("@title", "What do you know about the crash?"));
     m_awarenessPage->setIcon(QIcon::fromTheme(QStringLiteral("checkbox")));
 
@@ -77,30 +83,36 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
     connectSignals(m_backtrace);
 
     auto *m_backtracePage = new KPageWidgetItem(m_backtrace, QLatin1String(PAGE_CRASHINFORMATION_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_CRASHINFORMATION_ID), m_backtracePage);
     m_backtracePage->setHeader(i18nc("@title", "Fetching the Backtrace (Automatic Crash Information)"));
     m_backtracePage->setIcon(QIcon::fromTheme(QStringLiteral("run-build")));
 
     //-Results Page
     auto *m_conclusions = new ConclusionPage(this);
     connectSignals(m_conclusions);
-
-    auto *m_conclusionsPage = new KPageWidgetItem(m_conclusions, QLatin1String(PAGE_CONCLUSIONS_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_CONCLUSIONS_ID), m_conclusionsPage);
-    m_conclusionsPage->setHeader(i18nc("@title", "Results of the Analyzed Crash Details"));
-    m_conclusionsPage->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
     connect(m_conclusions, &ConclusionPage::finished, this, &ReportAssistantDialog::assistantFinished);
+    const QString conclusionHeader = i18nc("@title", "Results of the Analyzed Crash Details");
+    const QIcon conclusionIcon = QIcon::fromTheme(QStringLiteral("dialog-information"));
 
-    // Version check page
-    auto *versionPage = new BugzillaVersionPage(this);
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZVERSION_ID), versionPage->item());
+    // We have another of possible points where the conclusion page may appear. They are all individual pageitems
+    // refering to the same page. They'll be skipped so long as the page itself doesn't become isAppropriate() at any
+    // point.
+    auto *m_conclusionsPageAwareness = new KPageWidgetItem(m_conclusions, QLatin1String(PAGE_CONCLUSIONS_ID));
+    m_conclusionsPageAwareness->setHeader(conclusionHeader);
+    m_conclusionsPageAwareness->setIcon(conclusionIcon);
+
+    auto *m_conclusionsPageBacktrace = new KPageWidgetItem(m_conclusions, QLatin1String(PAGE_CONCLUSIONS_ID));
+    m_conclusionsPageBacktrace->setHeader(conclusionHeader);
+    m_conclusionsPageBacktrace->setIcon(conclusionIcon);
+
+    auto *m_conclusionsPageDuplicate = new KPageWidgetItem(m_conclusions, QLatin1String(PAGE_CONCLUSIONS_ID));
+    m_conclusionsPageDuplicate->setHeader(conclusionHeader);
+    m_conclusionsPageDuplicate->setIcon(conclusionIcon);
 
     //-Bugzilla Login
     auto *m_bugzillaLogin = new BugzillaLoginPage(this);
     connectSignals(m_bugzillaLogin);
 
     auto *m_bugzillaLoginPage = new KPageWidgetItem(m_bugzillaLogin, QLatin1String(PAGE_BZLOGIN_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZLOGIN_ID), m_bugzillaLoginPage);
     m_bugzillaLoginPage->setHeader(i18nc("@title", "Login into %1", i18n(KDE_BUGZILLA_DESCRIPTION)));
     m_bugzillaLoginPage->setIcon(QIcon::fromTheme(QStringLiteral("user-identity")));
     connect(m_bugzillaLogin, &BugzillaLoginPage::loggedTurnToNextPage, this, &ReportAssistantDialog::loginFinished);
@@ -110,7 +122,6 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
     connectSignals(m_bugzillaDuplicates);
 
     auto *m_bugzillaDuplicatesPage = new KPageWidgetItem(m_bugzillaDuplicates, QLatin1String(PAGE_BZDUPLICATES_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZDUPLICATES_ID), m_bugzillaDuplicatesPage);
     m_bugzillaDuplicatesPage->setHeader(i18nc("@title", "Look for Possible Duplicate Reports"));
     m_bugzillaDuplicatesPage->setIcon(QIcon::fromTheme(QStringLiteral("repository")));
 
@@ -119,7 +130,6 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
     connectSignals(m_bugzillaInformation);
 
     auto *m_bugzillaInformationPage = new KPageWidgetItem(m_bugzillaInformation, QLatin1String(PAGE_BZDETAILS_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZDETAILS_ID), m_bugzillaInformationPage);
     m_bugzillaInformationPage->setHeader(i18nc("@title", "Enter the Details about the Crash"));
     m_bugzillaInformationPage->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
 
@@ -127,7 +137,6 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
     auto *m_bugzillaPreview = new BugzillaPreviewPage(this);
 
     auto *m_bugzillaPreviewPage = new KPageWidgetItem(m_bugzillaPreview, QLatin1String(PAGE_BZPREVIEW_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZPREVIEW_ID), m_bugzillaPreviewPage);
     m_bugzillaPreviewPage->setHeader(i18nc("@title", "Preview the Report"));
     m_bugzillaPreviewPage->setIcon(QIcon::fromTheme(QStringLiteral("document-preview")));
 
@@ -135,19 +144,21 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget *parent)
     auto *m_bugzillaSend = new BugzillaSendPage(this);
 
     auto *m_bugzillaSendPage = new KPageWidgetItem(m_bugzillaSend, QLatin1String(PAGE_BZSEND_ID));
-    m_pageWidgetMap.insert(QLatin1String(PAGE_BZSEND_ID), m_bugzillaSendPage);
     m_bugzillaSendPage->setHeader(i18nc("@title", "Sending the Crash Report"));
     m_bugzillaSendPage->setIcon(QIcon::fromTheme(QStringLiteral("applications-internet")));
     connect(m_bugzillaSend, &BugzillaSendPage::finished, this, &ReportAssistantDialog::assistantFinished);
 
     // Need to append because the welcome page is conditionally pushed early on.
     m_pageItems.insert(m_pageItems.end(),
-                       {m_awarenessPage,
+                       {versionPage->item(),
+                        entityPage->item(),
+                        m_awarenessPage,
+                        m_conclusionsPageAwareness,
                         m_backtracePage,
-                        m_conclusionsPage,
-                        versionPage->item(),
+                        m_conclusionsPageBacktrace,
                         m_bugzillaLoginPage,
                         m_bugzillaDuplicatesPage,
+                        m_conclusionsPageDuplicate,
                         m_bugzillaInformationPage,
                         m_bugzillaPreviewPage,
                         m_bugzillaSendPage});
@@ -197,69 +208,28 @@ void ReportAssistantDialog::currentPageChanged_slot(KPageWidgetItem *current, KP
         beforePage->aboutToHide();
     }
 
-    // Load data of the current(new) page
-    if (current) {
-        auto *currentPage = qobject_cast<ReportAssistantPage *>(current->widget());
+    qCDebug(DRKONQI_LOG) << "Moving: " << (before ? before->name() : QString()) << "->" << current->name();
 
-        if (!currentPage->isAppropriate()) {
-            // The page is inappropriate. Find where to go next. This is extra exhausting because
-            // we can't just find the next or previous appropriate page because next() and back() implement lots of
-            // bespoke movement rules. So we in fact constantly need to go through those functions and then check again here.
-            // WARNING: when no page is appropriate this produces an ininfinite loop. There is no way to
-            //   prevent this right now. next() and back() would need to stop hacking in shortcuts for us to reliably
-            //   determine where to go and when we can't go anywhere.
-            enum class Movement { Unknown, Forward, Backward };
-            auto move = Movement::Unknown;
-            for (auto it = m_pageItems.cbegin(); it != m_pageItems.cend(); ++it) {
-                if (*it == before && move == Movement::Unknown) {
-                    // Found previous item first, we are moving forward ->
-                    move = Movement::Forward;
-                    continue; // continue finding current item, depending on where it is in the container we may need to reverse
-                }
-                if (*it == current) {
-                    if (move == Movement::Unknown) {
-                        // Current item first, we are moving backward <-
-                        move = Movement::Backward;
-                        if (it == m_pageItems.cbegin()) {
-                            // Turn around if this item is first but inappropriate.
-                            move = Movement::Forward;
-                        }
-                    } else if ((it + 1) == m_pageItems.cend()) {
-                        // Turn around if this item is last but inappropriate.
-                        move = Movement::Backward;
-                    }
-                    break;
-                }
-            }
+    if (!current) {
+        Q_UNREACHABLE(); // oughtn't ever happen. We always have a page to show so we always have a current page.
+        return;
+    }
 
-            qCDebug(DRKONQI_LOG) << "page inappropriate, skipping";
+    auto *currentPage = qobject_cast<ReportAssistantPage *>(current->widget());
 
-            switch (move) {
-            case Movement::Unknown:
-                qCDebug(DRKONQI_LOG) << "unknown";
-                Q_UNREACHABLE();
-                Q_FALLTHROUGH(); // do whatever at this point but do something, the page is inappropriate.
-            case Movement::Forward:
-                qCDebug(DRKONQI_LOG) << "forward";
-                next();
-                return;
-            case Movement::Backward:
-                qCDebug(DRKONQI_LOG) << "backward";
-                back();
-                return;
-            }
-            return;
-        }
+    if (!currentPage->isAppropriate()) {
+        skipInappropriatePage();
+        return;
+    }
 
-        nextButton()->setEnabled(currentPage->isComplete());
-        currentPage->aboutToShow();
+    nextButton()->setEnabled(currentPage->isComplete());
+    currentPage->aboutToShow();
 
-        // If the current page is the last one, disable all the buttons until the bug is sent
-        if (current->name() == QLatin1String(PAGE_BZSEND_ID)) {
-            nextButton()->setEnabled(false);
-            backButton()->setEnabled(false);
-            finishButton()->setEnabled(false);
-        }
+    // If the current page is the last one, disable all the buttons until the bug is sent
+    if (current == m_pageItems.back()) {
+        nextButton()->setEnabled(false);
+        backButton()->setEnabled(false);
+        finishButton()->setEnabled(false);
     }
 }
 
@@ -306,6 +276,7 @@ void ReportAssistantDialog::showHelp()
 // Override KAssistantDialog "next" page implementation
 void ReportAssistantDialog::next()
 {
+    m_move = Movement::Forward;
     // FIXME: this entire function is a bit weird. It'd likely make more sense to
     // use the page appropriateness more globally. i.e. mark pages inappropriate
     // when they are not applicable based on earlier settings done (e.g. put
@@ -318,45 +289,7 @@ void ReportAssistantDialog::next()
     if (page && !page->showNextPage()) {
         return;
     }
-
-    const QString name = currentPage()->name();
-
-    // If the information the user can provide is not useful, skip the backtrace page
-    if (name == QLatin1String(PAGE_AWARENESS_ID)) {
-        // Force save settings in the current page
-        page->aboutToHide();
-
-        if (!(m_reportInterface->isBugAwarenessPageDataUseful())) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_CONCLUSIONS_ID)));
-            return;
-        }
-    } else if (name == QLatin1String(PAGE_CRASHINFORMATION_ID)) {
-        // Force save settings in current page
-        page->aboutToHide();
-
-        // Qt aborts clients when the wayland compositor crashes. We can't do anything with these reports and immediately
-        // jump to the conclusion page. Additional handling happens there.
-        if (DrKonqi::debuggerManager()->backtraceGenerator()->parser()->hasCompositorCrashed()) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_CONCLUSIONS_ID)));
-            return;
-        }
-
-        // If the crash is worth reporting and it is BKO, skip the Conclusions page
-        if (m_reportInterface->isWorthReporting() && DrKonqi::crashedApplication()->bugReportAddress().isKdeBugzilla()) {
-            // Depending on whether the page is appropriate either go to version
-            // check page or login page.
-            const auto versionPage = m_pageWidgetMap.value(QLatin1String(PAGE_BZVERSION_ID));
-            const auto loginPage = m_pageWidgetMap.value(QLatin1String(PAGE_BZLOGIN_ID));
-            setCurrentPage(isAppropriate(versionPage) ? versionPage : loginPage);
-            return;
-        }
-    } else if (name == QLatin1String(PAGE_BZDUPLICATES_ID)) {
-        // a duplicate has been found, yet the report is not being attached
-        if (m_reportInterface->duplicateId() && !m_reportInterface->attachToBugNumber()) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_CONCLUSIONS_ID)));
-            return;
-        }
-    }
+    page->aboutToHide(); // save current page
 
     KAssistantDialog::next();
 }
@@ -365,24 +298,7 @@ void ReportAssistantDialog::next()
 // It has to mirror the custom next() implementation
 void ReportAssistantDialog::back()
 {
-    if (currentPage()->name() == QLatin1String(PAGE_CONCLUSIONS_ID)) {
-        if (m_reportInterface->duplicateId() && !m_reportInterface->attachToBugNumber()) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_BZDUPLICATES_ID)));
-            return;
-        }
-        if (!(m_reportInterface->isBugAwarenessPageDataUseful())) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_AWARENESS_ID)));
-            return;
-        }
-    }
-
-    if (currentPage()->name() == QLatin1String(PAGE_BZLOGIN_ID)) {
-        if (m_reportInterface->isWorthReporting() && DrKonqi::crashedApplication()->bugReportAddress().isKdeBugzilla()) {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_CRASHINFORMATION_ID)));
-            return;
-        }
-    }
-
+    m_move = Movement::Backward;
     KAssistantDialog::back();
 }
 
@@ -450,3 +366,36 @@ void ReportAssistantDialog::closeEvent(QCloseEvent *event)
         event->accept();
     }
 }
+
+void ReportAssistantDialog::skipInappropriatePage()
+{
+    qCDebug(DRKONQI_LOG) << "page inappropriate, skipping...";
+
+    switch (m_move) {
+    case Movement::Unknown:
+        qCDebug(DRKONQI_LOG) << "  ...unknown direction!";
+        Q_UNREACHABLE();
+        return; // do whatever at this point but do something, the page is inappropriate.
+    case Movement::Forward:
+        qCDebug(DRKONQI_LOG) << "  ...forward";
+        next();
+        return;
+    case Movement::Backward:
+        qCDebug(DRKONQI_LOG) << "   ...backward";
+        back();
+        return;
+    }
+}
+
+bool ReportAssistantDialog::isItemAppropriate(KPageWidgetItem *item) const
+{
+    // The base classes isAppropriate is not suitable for what we want, so we have a separate appropriateness system.
+    // This function helps with getting our appropriateness value. It's named differently from the same function
+    // in the base class, so we can at least try and intercept incorrect calls to that other function. It's hit and
+    // miss though since the function isn't virtual. Alas.
+    if (!item) {
+        return false;
+    }
+    auto page = qobject_cast<ReportAssistantPage *>(item->widget());
+    return page && page->isAppropriate();
+}
